local CowTip = _G.CowTip
local CowTip_LuaText = CowTip:GetModule("LuaText")

local L = setmetatable({}, { __index = function(self, index) return index end })

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local function escape(text)
	return text:replace("|","||")
end

local function unescape(text)
	return text:replace("||", "|")
end

local function setLineText(info, value)
	local side = info[#info]
	local num = info.arg
	CowTip_LuaText.db.profile[side][num] = unescape(value)
end

local function getLineText(info)
	local side = info[#info]
	local num = info.arg
	return escape(CowTip_LuaText.db.profile[side][num])
end

local function makeLineOption(options, num)
	options.args["line" .. num] = {
		type = 'group',
		name = L["Line %d"]:format(num),
		desc = L["Line"],
		inline = true,
		order = num + 100,
		args = {
			lefts = {
				type = 'input',
				name = L["left"],
				desc = L["left text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 1,
			},
			rights = {
				type = 'input',
				name = L["right"],
				desc = L["right text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 2,
			},
		},
	}
end

local function showConfig()
	AceConfigDialog:SetDefaultSize("Cowtip_LuaText", 775, 500)
	AceConfigDialog:Open("Cowtip_LuaText")
	-- InterfaceOptionsFrame:Hide() -- hides everything...
end

function setOption(info, value)
	local key = info[#info]
	CowTip_LuaText.db.profile[key] = value
end
function getOption(info)
	local key = info[#info]
	return CowTip_LuaText.db.profile[key]
end

local options
function CowTip_LuaText:CreateOptions()
	if options then
		return
	end
	options = {
		name = L["CowTip LuaText"],
		desc = L[""],
		type = 'group',
		set = setOption,
		get = getOption,
		args = {
			alwaysShowLefts = {
				type = 'toggle',
				name = L["Always show texts"],
				desc = L["Always show left texts, even if the right text is empty"],
				order = 1,
			},
			handleErrors = {
				type = 'toggle',
				name = L["Handle text's lua-errors"],
				desc = L["Whether to handle lua-errors caused by custom texts. Handling the errors may take up more CPU."],
				order = 2,
				set = function(info, value)
						setOption(info, value)
						CowTip_LuaText:UpdateErrorHandler()
					end,
			},
			printErrors = {
				type = 'toggle',
				name = L["Show Errors"],
				desc = L["Print errors to Chatframe"],
				order = 3,
				disabled = function() return not CowTip_LuaText.db.profile.handleErrors end,
			},
		},
	}
	local dbp = self.db.profile
	for i = 1,30 do
		makeLineOption(options, i)
	end

	AceConfig:RegisterOptionsTable("Cowtip_LuaText", options)

	local options2 = {
		name = L["CowTip LuaText"],
		desc = L[""],
		type = 'group',
		args = {
			showConfig = {
				name = L["Open Configuration"],
				desc = L["Open standalone Configuration"],
				type = 'execute',
				func = showConfig,
			},
		},
	}
	
	AceConfig:RegisterOptionsTable("Cowtip_LuaText_bliz", options2)
	AceConfigDialog:AddToBlizOptions("Cowtip_LuaText_bliz", "Cowtip LuaText")
end
